/*
    eonlink.c

    Downloads data from a Suunto EON Dive Computer

    Copyright (C) 2001 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  

*/

#include <fcntl.h>
#include <sys/termios.h>
#include <sys/ioctl.h>
#include <stdio.h>

/*
 *  Function prototypes
 */

unsigned char * progressbar(int length, int max, int curr);
int openlink(char * device);

int main(int argc, char **argv) {
  
  char *device = DEVICE;
  int fd, fd2;
  int rec;
  int count;
  unsigned char buf[1];
  unsigned char temp[255];
  unsigned char * progress;

  fd = openlink(device);
  
   if (fd < 1) {
     printf("Error opening serial port!!");
     return(-1);
   }

   printf("\nPut your EON into LOG mode before starting this program.\n");
   printf("To abort press Ctrl-C !\n\n");

   fd2=open("eondump.raw",O_WRONLY | O_CREAT | O_TRUNC, 0644);

   temp[0]='P';
   write(fd,temp,1);
   count=0;
   
   printf("Downloading data\n\n");
   
   while(count<2305) {
     rec=0;
     rec=read(fd,buf,1);
     if (rec>0) {
       count+=rec;

       if (count % 5 ==0) {
         progress = progressbar(70,2305,count);
         printf("\r%s", progress);
       }

       write(fd2, &buf ,1);
     }
   }
   close(fd);
   close(fd2);
   return(0);
}

int openlink(char * device) {
  struct termios settings;
  int fd;
  int modelines=0;

   fd=open(device, O_RDWR|O_NOCTTY);
   if(fd==-1) {
     fprintf(stderr,"Device %s cannot be opened!\n",device);
     return(fd);
   }

   if(!isatty(fd)) {
     fprintf(stderr,"Device %s is not a terminal-device !\n",device);
     return(-1);
   }
   
   /*Set inputflags*/
   settings.c_iflag=   IGNBRK  /* Ignore Input breaks */
                     | IGNPAR; /* No Parity  */


   /*Set outputflags*/
   settings.c_oflag=0;         /* nothing special */

   /*Set controlflags*/
   settings.c_cflag= CS8     /* 8 bits per byte */
                   | CREAD   /* enable receiver */
                   | CLOCAL  /* Ignore modem status, line is local */
                   | CSTOPB; /* 2 stoppbits */

   /*Set local flags*/
   settings.c_lflag=0;         /* nothing special */  

   /*Set timeout*/
   settings.c_cc[VTIME]=20;  /* set timeout */

   /*Set minimal bytes to read*/
   settings.c_cc[VMIN]=1;
   
   /*Set baud rate*/
   if(cfsetspeed(&settings,B1200)!=0) {
     fprintf(stderr,"Failed to set baud rate on device %s !\n" ,device);
     return(-1);
   }

   /*Set parameters defined above to the serial device*/
   if(tcsetattr(fd,TCSANOW,&settings)==-1) {
     fprintf(stderr,"Failed to set terminal attributes on device %s !\n",device);
     return(-1);
   }

   /*Read io-controll parameters*/
   if(ioctl(fd,TIOCMGET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMGET on device %s!\n",device);
     return(-1);
   }

   /*Set RTS*/
   modelines &= ~TIOCM_RTS;
   if(ioctl(fd,TIOCMSET,&modelines)==-1) {
     fprintf(stderr,"Error doing ioctl TIOCMSET on device %s !\n",device); 
     return(-1);
   }
  return(fd);
}

unsigned char * progressbar(int length, int max, int curr) {
  static unsigned char buf[255];
  unsigned char bar[255];
  int i, proc, barlength;
  
  buf[0]=0;
  bar[0]=0;
  
  if (curr == 0) {
    proc = 0;
    barlength = 0;
  } else {
    proc = (int) (curr * 100.0 / max);
    barlength = (int) ((curr * 1.0/ max * 1.0) * (length-7.0));
  }

  for(i=1; i<=(length-7); i++) {
    if ( i<=barlength ) {
      strcat(bar, "#");
    } else {
      strcat(bar, "-");
    }
  }

  sprintf(buf, "|%s| %03d%%", bar, proc);
  return(buf);

}
